==> graviti-0.2.3-py3-none-any.whl: graviti/utility/tbrn.py <==
#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file defines TensorBay Resource Name (TBRN) related classes."""

from enum import Enum, Flag, auto
from typing import Any, List, Optional, Tuple, TypeVar


class _TBRNFlag(Flag):
    FIELD_HEAD = 1 << 0
    FIELD_DATASET = 1 << 1
    FIELD_SEGMENT = 1 << 2
    FIELD_FRAME = 1 << 3
    FIELD_SENSOR = 1 << 4
    FIELD_PATH = 1 << 5

    # tb:[dataset]
    DATASET = FIELD_HEAD | FIELD_DATASET

    # tb:[dataset]:[segment]
    SEGMENT = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT

    # tb:[dataset]:[segment]:[frame]
    FRAME = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME

    # tb:[dataset]:[segment]::[sensor]
    SEGMENT_SENSOR = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_SENSOR

    # tb:[dataset]:[segment]:[frame]:[sensor]
    FRAME_SENSOR = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME | FIELD_SENSOR

    # tb:[dataset]:[segment]://[remote_path]
    NORMAL_FILE = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_PATH

    # tb:[dataset]:[segment]:[frame]:[sensor]://[remote_path]
    FUSION_FILE = (
        FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME | FIELD_SENSOR | FIELD_PATH
    )


class TBRNType(Enum):
    """`TBRNType` represent the type of a TBRN, which has 6 types:
    1. `TBRNType.DATASET`:
        "tb:VOC2012"
        which means the dataset "VOC2012"
    2. `TBRNType.SEGMENT`:
        "tb:VOC2010:train"
        which means the "train" segment of dataset "VOC2012"
    3. `TBRNType.FRAME`:
        "tb:KITTI:test:10"
        which means the 10th frame of the "test" segment in dataset "KITTI"
    4. `TBRNType.SEGMENT_SENSOR`:
        "tb:KITTI:test::lidar"
        which means the sensor "lidar" of the "test" segment in dataset "KITTI"
    5. `TBRNType.FRAME_SENSOR`:
        "tb:KITTI:test:10:lidar"
        which means the sensor "lidar" which belongs to the 10th frame of the "test" segment in
        dataset "KITTI"
    6. `TBRNType.NORMAL_FILE`:
        "tb:VOC2012:train://2012_004330.jpg"
        which means the file "2012_004330.jpg" of the "train" segment in normal dataset "VOC2012"
    7. `TBRNType.FUSION_FILE`
        "tb:KITTI:test:10:lidar://000024.bin"
        which means the file "000024.bin" in fusion dataset "KITTI", its segment, frame index and
        sensor is "test", 10 and "lidar"
    """

    DATASET = auto()
    SEGMENT = auto()
    FRAME = auto()
    SEGMENT_SENSOR = auto()
    FRAME_SENSOR = auto()
    NORMAL_FILE = auto()
    FUSION_FILE = auto()


T = TypeVar("T")  # pylint: disable=invalid-name


class TBRN:
    """`TBRN` is a TensorBay Resource Name(TBRN) parser and generator.

    Use as a generator:
        >>> info = TBRN("VOC2010", "train", remote_path="2012_004330.jpg")
        >>> info.type
        <TBRNType.NORMAL_FILE: 5>
        >>> info.get_tbrn()
        'tb:VOC2010:train://2012_004330.jpg'
        >>> print(info)
        'tb:VOC2010:train://2012_004330.jpg'

    Use as a parser:
        >>> tbrn = "tb:VOC2010:train://2012_004330.jpg"
        >>> info = TBRN(tbrn=tbrn)
        >>> info.dataset
        'VOC2010'
        >>> info.segment_name
        'train'
        >>> info.remote_path
        ''2012_004330.jpg''

    :param dataset_name: name of the dataset.
    :param segment_name: name of the segment.
    :param frame_index: index of the frame.
    :param sensor_name: name of the sensor.
    :param remote_path: object path of the file.
    :param tbrn: full TBRN string
    :raises
        `TypeError`: The TBRN is invalid

    """

    _HEAD = "tb"
    _NAMES_SEPARATOR = ":"
    _NAMES_MAX_SPLIT = 4
    _PATH_SEPARATOR = "://"
    _PATH_MAX_SPLIT = 1
    _FRAME_INDEX = 3

    _FLAG_TO_TYPE = {
        _TBRNFlag.DATASET: (TBRNType.DATASET, 1),
        _TBRNFlag.SEGMENT: (TBRNType.SEGMENT, 2),
        _TBRNFlag.FRAME: (TBRNType.FRAME, 3),
        _TBRNFlag.SEGMENT_SENSOR: (TBRNType.SEGMENT_SENSOR, 4),
        _TBRNFlag.FRAME_SENSOR: (TBRNType.FRAME_SENSOR, 4),
        _TBRNFlag.NORMAL_FILE: (TBRNType.NORMAL_FILE, 2),
        _TBRNFlag.FUSION_FILE: (TBRNType.FUSION_FILE, 4),
    }

    _names: Tuple[
        Optional[str],
        Optional[str],
        Optional[int],
        Optional[str],
        Optional[str],
    ]

    def __init__(
        self,
        dataset_name: Optional[str] = None,
        segment_name: Optional[str] = None,
        frame_index: Optional[int] = None,
        sensor_name: Optional[str] = None,
        *,
        remote_path: Optional[str] = None,
        tbrn: Optional[str] = None,
    ) -> None:
        if tbrn is not None:
            splits = tbrn.split(TBRN._PATH_SEPARATOR, TBRN._PATH_MAX_SPLIT)
            names: List[Any] = splits[0].split(TBRN._NAMES_SEPARATOR, TBRN._NAMES_MAX_SPLIT)

            if names[0] != TBRN._HEAD:
                raise TypeError('TensorBay Resource Name should startwith "tb:"')

            names += [None] * (TBRN._NAMES_MAX_SPLIT + 1 - len(names))
            frame_index = names[TBRN._FRAME_INDEX]
            names[TBRN._FRAME_INDEX] = int(frame_index) if frame_index else None

            if len(splits) == TBRN._PATH_MAX_SPLIT + 1:
                names.append(splits[1])
            else:
                names.append(None)

            self._names = tuple(names[1:])  # type: ignore[assignment]

        else:
            self._names = (
                dataset_name,
                segment_name,
                frame_index,
                sensor_name,
                remote_path,
            )

        try:
            self._type, self._field_length = self._check_type()
        except KeyError as error:
            raise TypeError("Invalid TensorBay Resource Name") from error

    def _check_type(self) -> Tuple[TBRNType, int]:
        # https://github.com/PyCQA/pylint/issues/2224
        flag = _TBRNFlag.FIELD_HEAD.value  # pylint: disable=no-member
        for i, name in enumerate(self._names):
            if name is not None:
                flag |= 1 << i + 1

        return TBRN._FLAG_TO_TYPE[_TBRNFlag(flag)]

    def __str__(self) -> str:
        return self.get_tbrn()

    def __repr__(self) -> str:
        return self.__str__()

    def _raise_when_none(self, value: Optional[T], field: str) -> T:
        if value is None:
            raise ValueError(f"TBRNType: '{self.type.name}' has no property '{field}'")
        return value

    @property
    def dataset_name(self) -> str:
        """Returns the dataset name.

        :return: the dataset name.
        """
        return self._names[0]  # type: ignore[return-value]

    @property
    def segment_name(self) -> str:
        """Returns the segment name.

        :return: the segment name.
        """
        return self._raise_when_none(self._names[1], "segment_name")

    @property
    def frame_index(self) -> int:
        """Returns the frame index.

        :return: the frame index.
        """
        return self._raise_when_none(self._names[2], "frame_index")

    @property
    def sensor_name(self) -> str:
        """Returns the sensor name.

        :return: the sensor name.
        """
        return self._raise_when_none(self._names[3], "sensor_name")

    @property
    def remote_path(self) -> str:
        """Returns the object path.

        :return: the object path.
        """
        return self._raise_when_none(self._names[4], "remote_path")

    @property
    def type(self) -> TBRNType:
        """Returns the type of this TBRN.

        :return: the type of this TBRN.
        """
        return self._type

    def get_tbrn(self, frame_width: int = 0) -> str:
        """Generate the full TBRN string
        :param frame_width: will adds '0' at the beginning of the frame_index,
            until it reaches the frame_width.

        :return: the full TBRN string
        """
        names: List[Any] = [TBRN._HEAD]
        names.extend(self._names[: self._field_length])
        if self._field_length >= TBRN._FRAME_INDEX:
            frame_index = names[TBRN._FRAME_INDEX]
            if frame_index is not None:
                if frame_width:
                    names[TBRN._FRAME_INDEX] = f"{frame_index:0{frame_width}}"
                else:
                    names[TBRN._FRAME_INDEX] = f"{frame_index}"
            else:
                names[TBRN._FRAME_INDEX] = ""
        tbrn = TBRN._NAMES_SEPARATOR.join(names)
        if self._names[4] is not None:
            tbrn = f"{tbrn}{TBRN._PATH_SEPARATOR}{self.remote_path}"
        return tbrn

==> graviti-0.3.8-py3-none-any.whl: graviti/utility/tbrn.py <==
#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file defines TensorBay Resource Name (TBRN) related classes."""

from enum import Enum, Flag, auto
from typing import Any, List, Optional, Tuple, TypeVar


class _TBRNFlag(Flag):
    FIELD_HEAD = 1 << 0
    FIELD_DATASET = 1 << 1
    FIELD_SEGMENT = 1 << 2
    FIELD_FRAME = 1 << 3
    FIELD_SENSOR = 1 << 4
    FIELD_PATH = 1 << 5

    # tb:[dataset]
    DATASET = FIELD_HEAD | FIELD_DATASET

    # tb:[dataset]:[segment]
    SEGMENT = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT

    # tb:[dataset]:[segment]:[frame]
    FRAME = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME

    # tb:[dataset]:[segment]::[sensor]
    SEGMENT_SENSOR = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_SENSOR

    # tb:[dataset]:[segment]:[frame]:[sensor]
    FRAME_SENSOR = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME | FIELD_SENSOR

    # tb:[dataset]:[segment]://[remote_path]
    NORMAL_FILE = FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_PATH

    # tb:[dataset]:[segment]:[frame]:[sensor]://[remote_path]
    FUSION_FILE = (
        FIELD_HEAD | FIELD_DATASET | FIELD_SEGMENT | FIELD_FRAME | FIELD_SENSOR | FIELD_PATH
    )


class TBRNType(Enum):
    """`TBRNType` represent the type of a TBRN, which has 6 types:
    1. `TBRNType.DATASET`:
        "tb:VOC2012"
        which means the dataset "VOC2012"
    2. `TBRNType.SEGMENT`:
        "tb:VOC2010:train"
        which means the "train" segment of dataset "VOC2012"
    3. `TBRNType.FRAME`:
        "tb:KITTI:test:10"
        which means the 10th frame of the "test" segment in dataset "KITTI"
    4. `TBRNType.SEGMENT_SENSOR`:
        "tb:KITTI:test::lidar"
        which means the sensor "lidar" of the "test" segment in dataset "KITTI"
    5. `TBRNType.FRAME_SENSOR`:
        "tb:KITTI:test:10:lidar"
        which means the sensor "lidar" which belongs to the 10th frame of the "test" segment in
        dataset "KITTI"
    6. `TBRNType.NORMAL_FILE`:
        "tb:VOC2012:train://2012_004330.jpg"
        which means the file "2012_004330.jpg" of the "train" segment in normal dataset "VOC2012"
    7. `TBRNType.FUSION_FILE`
        "tb:KITTI:test:10:lidar://000024.bin"
        which means the file "000024.bin" in fusion dataset "KITTI", its segment, frame index and
        sensor is "test", 10 and "lidar"
    """

    DATASET = auto()
    SEGMENT = auto()
    FRAME = auto()
    SEGMENT_SENSOR = auto()
    FRAME_SENSOR = auto()
    NORMAL_FILE = auto()
    FUSION_FILE = auto()


T = TypeVar("T")  # pylint: disable=invalid-name


class TBRN:
    """`TBRN` is a TensorBay Resource Name(TBRN) parser and generator.

    Use as a generator:
        >>> info = TBRN("VOC2010", "train", remote_path="2012_004330.jpg")
        >>> info.type
        <TBRNType.NORMAL_FILE: 5>
        >>> info.get_tbrn()
        'tb:VOC2010:train://2012_004330.jpg'
        >>> print(info)
        'tb:VOC2010:train://2012_004330.jpg'

    Use as a parser:
        >>> tbrn = "tb:VOC2010:train://2012_004330.jpg"
        >>> info = TBRN(tbrn=tbrn)
        >>> info.dataset
        'VOC2010'
        >>> info.segment_name
        'train'
        >>> info.remote_path
        ''2012_004330.jpg''

    :param dataset_name: name of the dataset.
    :param segment_name: name of the segment.
    :param frame_index: index of the frame.
    :param sensor_name: name of the sensor.
    :param remote_path: object path of the file.
    :param tbrn: full TBRN string
    :raises
        `TypeError`: The TBRN is invalid

    """

    _HEAD = "tb"
    _NAMES_SEPARATOR = ":"
    _NAMES_MAX_SPLIT = 4
    _PATH_SEPARATOR = "://"
    _PATH_MAX_SPLIT = 1
    _FRAME_INDEX = 3

    _FLAG_TO_TYPE = {
        _TBRNFlag.DATASET: (TBRNType.DATASET, 1),
        _TBRNFlag.SEGMENT: (TBRNType.SEGMENT, 2),
        _TBRNFlag.FRAME: (TBRNType.FRAME, 3),
        _TBRNFlag.SEGMENT_SENSOR: (TBRNType.SEGMENT_SENSOR, 4),
        _TBRNFlag.FRAME_SENSOR: (TBRNType.FRAME_SENSOR, 4),
        _TBRNFlag.NORMAL_FILE: (TBRNType.NORMAL_FILE, 2),
        _TBRNFlag.FUSION_FILE: (TBRNType.FUSION_FILE, 4),
    }

    _names: Tuple[
        Optional[str],
        Optional[str],
        Optional[int],
        Optional[str],
        Optional[str],
    ]

    def __init__(
        self,
        dataset_name: Optional[str] = None,
        segment_name: Optional[str] = None,
        frame_index: Optional[int] = None,
        sensor_name: Optional[str] = None,
        *,
        remote_path: Optional[str] = None,
        tbrn: Optional[str] = None,
    ) -> None:
        if tbrn is not None:
            splits = tbrn.split(TBRN._PATH_SEPARATOR, TBRN._PATH_MAX_SPLIT)
            names: List[Any] = splits[0].split(TBRN._NAMES_SEPARATOR, TBRN._NAMES_MAX_SPLIT)

            if names[0] != TBRN._HEAD:
                raise TypeError('TensorBay Resource Name should startwith "tb:"')

            names += [None] * (TBRN._NAMES_MAX_SPLIT + 1 - len(names))
            frame_index = names[TBRN._FRAME_INDEX]
            names[TBRN._FRAME_INDEX] = int(frame_index) if frame_index else None

            if len(splits) == TBRN._PATH_MAX_SPLIT + 1:
                names.append(splits[1])
            else:
                names.append(None)

            self._names = tuple(names[1:])  # type: ignore[assignment]

        else:
            self._names = (
                dataset_name,
                segment_name,
                frame_index,
                sensor_name,
                remote_path,
            )

        try:
            self._type, self._field_length = self._check_type()
        except KeyError as error:
            raise TypeError("Invalid TensorBay Resource Name") from error

    def _check_type(self) -> Tuple[TBRNType, int]:
        # https://github.com/PyCQA/pylint/issues/2224
        flag = _TBRNFlag.FIELD_HEAD.value  # pylint: disable=no-member
        for i, name in enumerate(self._names, 1):
            if name is not None:
                flag |= 1 << i

        return TBRN._FLAG_TO_TYPE[_TBRNFlag(flag)]

    def __repr__(self) -> str:
        return self.get_tbrn()

    def _raise_when_none(self, value: Optional[T], field: str) -> T:
        if value is None:
            raise ValueError(f"TBRNType: '{self.type.name}' has no property '{field}'")
        return value

    @property
    def dataset_name(self) -> str:
        """Returns the dataset name.

        :return: the dataset name.
        """
        return self._names[0]  # type: ignore[return-value]

    @property
    def segment_name(self) -> str:
        """Returns the segment name.

        :return: the segment name.
        """
        return self._raise_when_none(self._names[1], "segment_name")

    @property
    def frame_index(self) -> int:
        """Returns the frame index.

        :return: the frame index.
        """
        return self._raise_when_none(self._names[2], "frame_index")

    @property
    def sensor_name(self) -> str:
        """Returns the sensor name.

        :return: the sensor name.
        """
        return self._raise_when_none(self._names[3], "sensor_name")

    @property
    def remote_path(self) -> str:
        """Returns the object path.

        :return: the object path.
        """
        return self._raise_when_none(self._names[4], "remote_path")

    @property
    def type(self) -> TBRNType:
        """Returns the type of this TBRN.

        :return: the type of this TBRN.
        """
        return self._type

    def get_tbrn(self, frame_width: int = 0) -> str:
        """Generate the full TBRN string
        :param frame_width: will adds '0' at the beginning of the frame_index,
            until it reaches the frame_width.

        :return: the full TBRN string
        """
        names: List[Any] = [TBRN._HEAD]
        names.extend(self._names[: self._field_length])
        if self._field_length >= TBRN._FRAME_INDEX:
            frame_index = names[TBRN._FRAME_INDEX]
            if frame_index is not None:
                if frame_width:
                    names[TBRN._FRAME_INDEX] = f"{frame_index:0{frame_width}}"
                else:
                    names[TBRN._FRAME_INDEX] = f"{frame_index}"
            else:
                names[TBRN._FRAME_INDEX] = ""
        tbrn = TBRN._NAMES_SEPARATOR.join(names)
        if self._names[4] is not None:
            tbrn = f"{tbrn}{TBRN._PATH_SEPARATOR}{self.remote_path}"
        return tbrn
